import base64
import json
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class Github(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Github

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        conf = self._config.github
        if not conf.username or \
                not conf.repo or \
                not conf.save_path or \
                not conf.token:
            return self._set_error('config is miss')
        # 获取文件名称
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        url = f"https://api.github.com/repos/{conf.username}/{conf.repo}/" \
              f"contents/{conf.save_path}/" \
              + file_name  # 用户名、库名、路径
        data = {
            "message": "upload file",
            "content": base64.b64encode(content).decode(),
            'branch': conf.branch
        }
        headers = {
            "Authorization": "token " + conf.token
        }
        response = requests.put(url=url, data=json.dumps(data), headers=headers)
        if response.status_code != 201 and response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        # 获取图片链接
        url = response.json().get('content', {}).get('download_url')
        if not url:
            return self._set_error(f'response error. {response.text}')
        # 返回结果
        if conf.use_jsdelivr:
            return f"https://cdn.jsdelivr.net/gh/{conf.username}/{conf.repo}@" \
                   f"{conf.branch}/{conf.save_path}/{file_name}"
        else:
            return url



from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class NiuPic(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.NiuPic

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        files = {
            'file': (file_name, content),
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/107.0.0.0 Safari/537.36',
            'origin': 'https://www.niupic.com',
            'referer': 'https://www.niupic.com/',
        }
        response = requests.post('https://www.niupic.com/api/upload', files=files, headers=headers)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        data = response.json()
        if data.get('code') != 200 or not data.get('data'):
            return self._set_error(response.text)
        return data.get('data')



from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class ImgUrl(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.ImgUrl

    def _upload(self, content) -> Optional[str]:
        conf = self._config.img_url
        if not conf.token or not conf.uid:
            return self._set_error('token or uid is empty')
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        files = {
            'file': (file_name, content)
        }
        data = {
            'uid': conf.uid,
            'token': conf.token,
        }
        response = requests.post('https://www.imgurl.org/api/v2/upload', files=files, data=data)
        if response.status_code != 200:
            return self._set_error(f'request fail. code: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'parse response fail. data: {response.text}')
        # 接口返回错误信息
        code = data.get('code')
        msg = data.get('msg')
        if code != 200 and msg:
            return self._set_error(msg)
        if code != 200:
            return self._set_error(f'response error. {response.text}')
        url = data.get('data', {}).get('url')
        if not url:
            return self._set_error(f'parse url error. {response.text}')
        return url



"""
@author hujing
"""
from enum import Enum, auto


class DepotType(Enum):
    CatBox = auto()
    """
    图床
    
    url: https://catbox.moe/
    
    image: https://files.catbox.moe/pzchfp
    """
    PomfSe = auto()
    """
    一类图床. 使用同一套框架运行, 上传接口一致. 
        * 不同人维护的不同服务.
        * 因此这里提供一个类型, 可配置上传连接 
        * 此服务可靠性应该是不高
    限制: 不同服务的限制不同
    
    配置: 
        * 可以不修改配置
        * 留出配置文件是为了防止所有服务都不能用了, 可以自定义上传接口
    
    可通过下方链接找到相似的站点,
        * 链接: https://pomf.se/
        * 上传参数: files[] (指定上传文件)
        * 响应: {"success": true, "files": [{"url": "xxx"}]} 
    
    注意: 
        * 默认配置的服务中存在会过期的图床, 使用时请确认
    
    url: https://pomf.se/
    
    image: https://s1.fileditch.ch/eUrajjLmzsKuXEDRzaBZ.jpg
    """
    PngCm = auto()
    """
    图床
    
    url: https://png.cm/
    
    image: https://i2.100024.xyz/2022/12/04/p5v5mj.webp
    """
    Riyugo = auto()
    """
    限制: 
        1. 图片3天内有效
        2. 单文件最大5M
        3. 免费版, 页面22:00 ~ 07:00 关闭上传功能
    url: https://zixiaoyun.com/
    
    image: https://beta.glilmu.com/i/2022/12/04/p6eo39.png
    """
    NiuPic = auto()
    """
    牛客网
    
    url: https://www.niupic.com/
    
    image: https://i.niupic.com/images/2022/12/04/abCy.png
    """
    ImgUrl = auto()
    """
    免费账户有上传限制:
        1. 每日限制上传15张图片
        2. 每月限制上传400张
    图片链接限制: 
        1. 浏览器直接打开图片时, 会重定向到图床网址
    url: https://www.imgurl.org/
    
    image: https://s3.uuu.ovh/imgs/2022/12/04/568047dc209095c7.jpg
    """
    SmMs = auto()
    """
    限制: 
        1. 免费账号限制存储空间(可注册多个账号)
        2. 图片链接空 referer 访问时, 会重定向到图床网址
        3. 每分钟上传限制20张
        4. 每日上传限制100张
    url: https://sm.ms/
    
    image: https://s2.loli.net/2022/12/04/XDt4H1zyMi6QvJ7.jpg
    """
    Imgbb = auto()
    """
    限制: 
        1. 图片链接空 referer 访问时, 会重定向到图床网址
    url: https://imgbb.com/
    
    image: https://i.ibb.co/Qnjwwy4/source.jpg
    """
    Xywm = auto()
    """
    图床
        1. 注册用户存在限制, 可能登录后查看
            * 限制并发数量及上传数量
    
    url: https://pic.xywm.ltd/
    
    image: https://p1.xywm.ltd/2022/12/04/638c4d21b9060.jpg
    """
    BoLuo = auto()
    """
    图床, 限制: 
        1. 每天限制上传50张, 每月上传1000张 (可自行付费扩容)
        2. 必须注册
    url: https://www.boluo.link/
    
    image: https://s1.boluo.link/imgs/2022/12/04/7bde14bb7169b7f2.jpg
    """
    Postimages = auto()
    """
    缺点: 
        1. 每次上传图片需要访问3次接口
        2. 图片链接空 referer 访问时, 会重定向到图床网址
    url: https://postimages.org/
    """
    Hakaimg = auto()
    """
    url: https://hakaimg.com/
    
    image: https://hakaimg.com/i/2022/12/04/pd0mkv.jpg
    """
    Tucang = auto()
    """
    图仓
    url: https://tucang.cc/
    
    image: https://img.tucang.cc/api/image/show/87917ba87bde14bb7169b7f225e430ac
    """
    ImgAx = auto()
    """
    url: https://img.ax/
    
    image: https://img.urlnode.com/file/ccd2064575362a738e943.jpg
    """
    HuaBan = auto()
    """
    花瓣网站的图片上传. 图片上传接口在图集上传处发现
    
    限制: 
        1. 需要先登录
        2. 图片做了 referer 判断, 空 referer 可访问
    url: https://huaban.com/
    
    image: https://gd-hbimg.huaban.com/670abb2c3e9db5ae69395775b5eb440730fdb4b43e39-viueHl
    """
    Ai58 = auto()
    """
    58同城客服, 在线上传图片
    
    url: https://ai.58.com
    
    image: https://pic3.58cdn.com.cn/nowater/webim/big/n_v2e0aa97ec21bb4f4581f365eff90a683c.jpg
    """
    Github = auto()
    """
    借用 github 仓库
    
    url: https://github.com
    
    image: https://cdn.jsdelivr.net/gh/hujingnb/img@test/content/img/f27ad12e-9bdb-4ab5-b2f7-91c0eff0ea55.jpg
    """
    PzAl = auto()
    """
    url: https://pz.al/
    
    image: https://f.pz.al/pzal/2023/11/19/37bd71a58ad86.jpg
    """

    '''============================已不再支持的图床, 保留是为了旧版本升级不报错'''
    Image23 = auto()
    """
    限制: 
        1. 单文件最大5M
    url: https://23img.com/

    image: https://23img.com/i/2023/10/04/w7igrm.jpg

    图床没了...
    """
    Dig268608 = auto()
    """
    url: https://www.268608.com/

    image: https://www.268608.com/imgs/2022/12/04/7bde14bb7169b7f2.jpg

    图床没了...
    """



import re
import time
import uuid
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class SmMs(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.BoLuo

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        conf = self._config.bo_luo
        if not conf.token or not conf.uid:
            return self._set_error('token is empty')
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        files = {
            'file': (file_name, content),
        }
        data = {
            'uid': conf.uid,
            'token': conf.token,
        }
        response = requests.post('https://www.boluo.link//api/v2/upload', files=files, data=data)
        if response.status_code != 200:
            return self._set_error(f'upload fail. status: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'content is error. {response.text}')
        # 上传失败
        if data.get('code') != 200 or not data.get('data'):
            return self._set_error(f'upload fail. {response.text}')
        url = data.get('data').get('url')
        if not url:
            return self._set_error(f'success but not image. {response.text}')
        return url



from typing import Optional
from .base import Depot


class NoneType(Depot):
    @classmethod
    def depot_type(cls) -> None:
        return None

    def __init__(self, type):
        super().__init__()
        self._type = type

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        return self._set_error(f'{self._type} not support or is delete')


"""
@author hujing
"""
from typing import Union


class ConfigBase:
    def __getattr__(self, name):
        """
        当属性未定义时会调用此方法, 直接返回 None, 防止报错
        :param name:
        :return:
        """
        return None

    def copy(self) -> 'ConfigBase':
        """
        当前类型深度克隆
        :return:
        """
        ret = self.__class__()
        for field in dir(self):
            if field.startswith('_'):
                continue
            value = getattr(self, field)
            if isinstance(value, ConfigBase):
                value = value.copy()
            setattr(ret, field, value)
        return ret


class DepotConfig(ConfigBase):
    def __getattr__(self, name):
        # 返回类
        # 当某个仓库失效时, 可以直接删除
        # 调用者仍然可以访问已删除的字段, 版本升级后不会报错
        return ConfigBase()

    class Smms(ConfigBase):
        """
        token 获取地址: https://sm.ms/home/apitoken \n
        """
        token: str = ''

    sm_ms = Smms()

    class Github(ConfigBase):
        username: str = ''  # 用户名
        repo: str = ''  # 仓库名称
        token: str = ''
        """
        获取地址: https://github.com/settings/tokens <br>
        创建 token 时, 权限选中 repo 的所有
        """
        save_path: str = ''  # 文件保存路径. eg: content/img
        use_jsdelivr: bool = True
        """
        是否使用 jsdelivr 加速 <br>
        若为 True, 则返回 jsdelivr 图片链接, 否则返回 github 链接 <br>
        默认为 True
        """
        branch: str = 'master'  # 指定分支名称, 分支必须存在. 默认'master'

    github = Github()

    class ImgUrl(ConfigBase):
        token: str = ''  # 获取地址: https://www.imgurl.org/vip/manage/mytoken
        uid: str = ''  # 与 token 一起获取

    img_url = ImgUrl()

    class HuaBan(ConfigBase):
        cookie: str = ''
        """
        登录网站 https://huaban.com 后
        将请求 header 中的 cookie 取出
        """

    hua_ban = HuaBan()

    class Xywm(ConfigBase):
        # key 获取地址: https://pic.xywm.ltd/settings/api
        api_key: str = ''

    xywm = Xywm()

    class PomfSe(ConfigBase):
        upload_url_list = [  # 修改此配置, 请确认您对需要的链接了解
            # https://fileditch.com/
            'https://up1.fileditch.com/upload.php',
            'https://qu.ax/upload.php',
            'https://midi.moe/upload.php',
            'https://cockfile.com/upload.php',
            'https://imouto.kawaii.su/api/upload',
            'https://safe.waifuhunter.club/api/upload',
            'https://take-me-to.space/api/upload',
            'https://files.htp.sh/api/upload',
            'https://pomf.lain.la/upload.php',
            'https://smutty.horse/upload.php',
            'https://stuff.poxydoxy.com/upload.php',
            # 48小时候自动过期
            'https://uguu.se/upload.php',
            # 24小时过期
            'https://cockfile.com/upload.php',
        ]

    pomf_se = PomfSe()

    class Tucang(ConfigBase):
        # 配置的获取请参考: http://doc.tucang.cc/project-1/doc-7/
        token: str = ''
        folder_id: str = 0

    tucang = Tucang()

    class BoLuo(ConfigBase):
        # 配置的获取请参考: https://www.boluo.link/vip/manage/mytoken
        token: str = ''
        uid: str = ''

    bo_luo = BoLuo()

    class PzAl(ConfigBase):
        # 非必填, 游客也可以上传, 但会有限制
        email: str = ''
        password: str = ''

    pz_al = PzAl()


_global_config = DepotConfig()


def get_config_by_dict(data: dict) -> Union[DepotConfig, ConfigBase]:
    """
    使用字典初始化配置
    外部配置可使用 yaml/json/xml 等不限, 自行转换为字典即可
    :param data:
    :return:
    """
    ret = DepotConfig()

    def set_config_by_dict(obj: ConfigBase, tmp_data: dict):
        for field in tmp_data:
            if not hasattr(obj, field):
                continue
            value = getattr(obj, field)
            if isinstance(value, ConfigBase):
                set_config_by_dict(value, tmp_data.get(field))
            else:
                setattr(obj, field, tmp_data.get(field))

    set_config_by_dict(ret, data)
    return ret


def set_global_config(config: DepotConfig):
    global _global_config
    _global_config = config


def get_global_config() -> DepotConfig:
    return _global_config



"""
定义仓库异常
@author hujing
"""
import traceback


class DepotError:
    def __init__(self, err, tb):
        """
        :param err:
        :param tb: 堆栈信息
        """
        if not tb:
            tb = traceback.extract_stack()[0:-1]
            tb = ''.join(traceback.format_list(tb))
        self._err = err
        self._traceback = tb

    def error(self):
        return self._err

    def traceback(self):
        return self._traceback

    def __str__(self):
        return self.error() + '\n' + \
            self.traceback()



import random
import re
import time
import uuid
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class SmMs(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.ImgAx

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        data = {
            'id': 'picId_' + self._random_str(8),
        }
        files = {
            "file": content
        }
        response = requests.post("https://img.ax/api.php", data=data, files=files)
        if response.status_code != 200:
            return self._set_error(f'upload fail. status: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'content is error. {response.text}')
        # 上传失败
        if data.get('code') != 200:
            return self._set_error(f'upload fail. {response.text}')
        # 上传成功, 但图片链接为空
        url = data.get('src')
        if not url:
            return self._set_error(f'success but not image. {response.text}')
        return url



from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class CatBox(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.CatBox

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        data = {
            'reqtype': 'fileupload',
            'userhash': '',
        }
        files = {
            'fileToUpload': content,
        }
        response = requests.post('https://catbox.moe/user/api.php', data=data, files=files)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        return response.text



from typing import Optional
from image_depot import DepotType
from .base import Depot
import requests


class HuaBan(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.HuaBan

    def _upload(self, content) -> Optional[str]:
        cookie = self._config.hua_ban.cookie
        if not cookie:
            return self._set_error('cookie is empty')
        files = {
            'file': content,
        }
        headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/107.0.0.0 Safari/537.36',
            'Cookie': cookie,
        }
        response = requests.post('https://api.huaban.com/upload', files=files, headers=headers)
        if response.status_code != 200:
            return self._set_error(f'request fail. code: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'parse response error. {response.text}')
        # 拼接图片链接
        bucket = data.get('bucket')
        key = data.get('key')
        if not bucket or not key:
            return self._set_error(f'upload fail. {response.text}')
        return f'https://{bucket}.huaban.com/{key}'



from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class PzAl(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.PzAl

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        # 获取上传 token
        token = None
        conf = self._config.pz_al
        if conf.email and conf.password:
            token = self._get_token(conf.email, conf.password)
        # 上传图片
        header = {}
        if token:
            header['token'] = token
        response = requests.post('https://pz.al/api/upload', headers=header, files={
            'image': (file_name, content),
        })
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        code = response.json().get('code')
        if code != 200:
            return self._set_error(f'upload fail. code: {code}. content: {response.text}')
        url = response.json().get('data', {}).get('url')
        if not url:
            return self._set_error(f'response error. {response.text}')
        return url

    def _get_token(self, email, password):
        response = requests.post('https://pz.al/api/token', data={
            'email': email,
            'password': password,
        })
        if response.status_code != 200:
            return self._set_error(f'get token fail. code: {response.status_code}. content: {response.text}')
        code = response.json().get('code')
        if code != 200:
            return self._set_error(f'get token fail. code: {code}. content: {response.text}')
        token = response.json().get('data', {}).get('token')
        if not token:
            return self._set_error(f'get token fail. token is empty')
        return token


from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class Tucang(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Tucang

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        conf = self._config.tucang
        if not conf.token:
            return self._set_error('token is empty')
        tmp_filename = self._random_file_name(content)
        if not tmp_filename:
            return None
        data = {
            'token': conf.token,
        }
        if conf.folder_id:
            data['folderId'] = conf.folder_id
        files = {
            "file": (tmp_filename, content)
        }
        response = requests.post("https://tucang.cc/api/v1/upload", files=files, data=data)
        if response.status_code != 200:
            return self._set_error(f'request fail. code: {response.status_code}')
        #  检查返回数据
        data = response.json()
        if not data.get('success'):
            return self._set_error(data.get('msg', response.text))
        url = data.get('data', {}).get('url')
        if not data.get('success') or not url:
            return self._set_error(f'server return error. {response.text}')
        return url



"""
@author hujing
"""
import os
from typing import List, Optional
from .depot import Depot, DEPOT_MAP, DepotError, DepotConfig
from .depot_type import DepotType
from .depot.none_type import NoneType


def image_depot(t: DepotType, config: Optional[DepotConfig] = None) -> Optional[Depot]:
    """
    根据 type 获取仓库对象
    :param config:
    :param t:
    :return:
    """
    if t in DEPOT_MAP:
        d = DEPOT_MAP[t]()
        if config:
            d.set_config(config)
        return d
    return NoneType(t)


def upload(content, type_list: List[DepotType] = None, config: Optional[DepotConfig] = None) -> (str, List[DepotError]):
    """
    按照指定的类型依次上传, 直到首次上传成功则返回
    若依次尝试后, 均失败, 则返回 None
    :param config:
    :param content:
    :param type_list: 若不指定, 则默认使用所有类型
    :return:
        str: 上传成功后的地址 <br>
        err: 若上传失败, 此数组保存所有失败原因
    """
    err = []
    if not type_list:
        type_list = DepotType
    for item in type_list:
        d = image_depot(item, config)
        ret = d.upload(content)
        if ret:
            return ret, None
        else:
            err.append(d.error())
    return None, err


def upload_file(file_path: str, type_list: List[DepotType] = None, config: Optional[DepotConfig] = None) \
        -> (str, List[DepotError]):
    if not os.path.exists(file_path):
        return None, [DepotError(f'file {file_path} not exist')]
    with open(file_path, 'rb') as f:
        return upload(f.read(), type_list, config)



import re
import time
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class Imgbb(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Imgbb

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        session = requests.session()
        # 访问首页, 获取token
        response = session.get('https://imgbb.com/')
        if response.status_code != 200:
            return self._set_error(f'request fail. code: {response.status_code}')
        re_search = re.search(r'auth_token\s*=\s*"([\S\d]+)"', response.text)
        if not re_search:
            return self._set_error('parse token fail')
        token = re_search.group(1)

        data = {
            'type': 'file',
            'action': 'upload',
            'timestamp': int(time.time() * 1000),
            'token': token,
        }
        files = {
            'source': content,
        }
        response = session.post('https://zh-cn.imgbb.com/json', data=data, files=files)
        if response.status_code != 200:
            return self._set_error(f'request fail. code: {response.status_code}')
        #  检查返回数据
        data = response.json()
        status_code = data.get('status_code')
        if status_code != 200:
            return self._set_error(f'server return error. {response.text}')
        url = data.get('image', {}).get('url')
        if not url:
            return self._set_error(f'parse url fail. {response.text}')
        return url



import base64
from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class Xywm(Depot):
    def __init__(self):
        super().__init__()
        self._token = None

    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Xywm

    def _upload(self, content) -> Optional[str]:
        conf = self._config.xywm
        if not conf.api_key:
            return self._set_error('api_key is empty')
        # 上传图片
        data = {
            'source': base64.b64encode(content).decode('utf-8'),
        }
        headers = {
            "Accept": "application/json",
            'X-API-Key': conf.api_key
        }
        response = requests.post("https://pic.xywm.ltd/api/1/upload", headers=headers, data=data)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}')
        data = response.json()
        if data.get('status_code') != 200:
            return self._set_error(f'upload fail. {response.text}')
        url = data.get('image', {}).get('image', {}).get('url')
        if not url:
            return self._set_error(f'response error. {response.text}')
        return url



import random
import time
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class PngCm(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.PngCm

    def _upload(self, content) -> Optional[str]:
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        data = {
            'uuid': 'o_' + self._random_str(27),
            'name': file_name,
            'sign': int(time.time()),
        }
        files = {
            'file': content,
        }
        response = requests.post('https://png.cm/app/upload.php', data=data, files=files)
        if response.status_code != 200:
            return self._set_error(f'upload fail. status: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'content is error. {response.text}')
        # 上传失败
        if data.get('result') != 'success':
            return self._set_error(f'upload fail. {response.text}')
        # 上传成功, 但图片链接为空
        url = data.get('url')
        if not url:
            return self._set_error(f'success but not image. {response.text}')
        return url



from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class SmMs(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.SmMs

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        conf = self._config.sm_ms
        if not conf.token:
            return self._set_error('token is empty')
        data = {
            'format': 'json',
        }
        headers = {
            'Authorization': conf.token,
        }
        files = {
            'smfile': content,
        }

        response = requests.post('https://smms.app/api/v2/upload', data=data, files=files, headers=headers, timeout=30)
        if response.status_code != 200:
            return self._set_error(f'upload fail. status: {response.status_code}')
        data = response.json()
        if not data:
            return self._set_error(f'content is error. {response.text}')
        # 上传失败
        if not data.get('success'):
            # 上传图片重复, 会将图片链接返回来
            if data.get('code') == 'image_repeated' and data.get('images'):
                return data.get('images')
            return self._set_error(f'upload fail. {response.text}')
        # 上传成功, 但图片链接为空
        url = data.get('data', {}).get('url')
        if not url:
            return self._set_error(f'success but not image. {response.text}')
        return url



import random
import re
from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class Riyugo(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Riyugo

    def _upload(self, content) -> Optional[str]:
        file_name = self._random_file_name(content)
        if not file_name:
            return None

        session = requests.session()
        session.headers = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/107.0.0.0 Safari/537.36',
        }
        # 通过首页获取 token
        response = session.get('https://zixiaoyun.com/')
        if response.status_code != 200:
            return self._set_error(f'get token fail. code: {response.status_code}')
        # authToken_today :"FJ932YTHEWOJG94YHEWJGOWEK349"
        # 从结果中通过正则匹配出 token
        matches = re.search(r"authToken_today\s*:\s*\"([0-9A-Za-z]+)\"", response.text)
        if not matches:
            return self._set_error(f'get token fail. content: {response.text}')
        token = matches.group(1)
        data = {
            'uuid': 'o_' + self._random_str(27),
            'nameMode': 'isRenameMode',
            'authToken_today': token,
            'name': file_name,
        }
        files = {
            'file': content,
        }
        headers = {
            'origin': 'https://zixiaoyun.com',
            'referer': 'https://zixiaoyun.com/',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/107.0.0.0 Safari/537.36',
        }
        response = session.post('https://s1.wzznft.com/localup.php', data=data, files=files, headers=headers)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        data = response.json()
        if data.get('result') != 'success' or not data.get('url'):
            return self._set_error(response.text)
        return data.get('url')



import os
import random
import traceback
import uuid
from abc import abstractmethod, ABC
from typing import Optional
import filetype
from image_depot import DepotType
from .depot_config import DepotConfig, get_global_config
from .error import DepotError


class Depot(ABC):
    _config: DepotConfig

    def __init__(self):
        self._err = None
        self._config = get_global_config()

    @classmethod
    @abstractmethod
    def depot_type(cls) -> DepotType:
        """
        当前类型
        :return:
        """
        pass

    def set_config(self, config: DepotConfig):
        """
        次函数用于自定义配置
        :return:
        """
        self._config = config

    # 上传图片, 二进制内容
    def upload(self, content) -> Optional[str]:
        try:
            return self._upload(content)
        except Exception as e:
            tb = traceback.format_exc()
            return self._set_error(e, tb)

    @abstractmethod
    def _upload(self, content) -> Optional[str]:
        pass

    # 上传图片, 本地路径
    def upload_file(self, file_path) -> Optional[str]:
        if not os.path.exists(file_path):
            return self._set_error(f'file {file_path} not exist')
        with open(file_path, 'rb') as f:
            return self.upload(f.read())

    def error(self) -> Optional[DepotError]:
        """
        获取错误信息
        :return:
        """
        if self._err:
            return self._err
        return None

    def _set_error(self, err, tb: str = '') -> None:
        if not tb:  # 未指定堆栈信息, 获取调用方
            tb = traceback.extract_stack()[0:-1]
            tb = ''.join(traceback.format_list(tb))
        self._err = DepotError(str(err), tb)
        return None

    def _random_str(self, length: int, rand_str: str = None):
        """
        获取指定长度的随机字符串
        :param length:
        :param rand_str:
        :return:
        """
        if not rand_str:
            rand_str = '0123456789abcdefghijklmnopqrstuvwxyz'
        return ''.join(random.sample(list(rand_str) * length, length))

    def _random_file_name(self, content):
        """
        获取随机的文件名, 会自动识别文件后缀
        :param content:
        :return:
        """
        # 创建临时文件, 获取文件后缀
        tmp_file = '/tmp/' + str(uuid.uuid4())
        with open(tmp_file, 'wb') as f:
            f.write(content)
        suffix = filetype.guess_extension(tmp_file)
        os.remove(tmp_file)
        if not suffix:
            return self._set_error(f'parse mimetype error.')
        return str(uuid.uuid4()) + '.' + suffix



import json
import re
import time
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class Postimages(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Postimages

    # 上传图片, 二进制内容
    def _upload(self, content) -> Optional[str]:
        # 访问主页获取 token
        response = requests.get('https://postimages.org/')
        if response.status_code != 200:
            return self._set_error(f'response fail. code: {response.status_code}')
        re_search = re.search(r'"token","(\w+)"', response.text)
        if not re_search:
            return self._set_error('parse token fail')
        token = re_search.group(1)
        # 上传图片
        data = {
            'token': token,
            'upload_session': self._random_str(32),
            'numfiles': 1,
            'ui': json.dumps(['', '', '', True, '', '', time.strftime("%Y/%m/%d %H:%M:%S")]),
            'optsize': '',
            'session_upload': int(time.time() * 1000),
            'gallery': '',
            'expire': 0,
        }
        files = {
            'file': content,
        }
        response = requests.post('https://postimages.org/json/rr', data=data, files=files)
        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        web_url = response.json().get('url')
        if not web_url:
            return self._set_error(f'parse error. {response.text}')
        # 获取图片链接
        response = requests.get(web_url)
        if response.status_code != 200:
            return self._set_error(f'got image url fail. code: {response.status_code}')
        re_search = re.search(r'meta\s*[\w="]+og:image[\s"\w=]+"([\w:/.]+)"', response.text)
        if not re_search:
            return self._set_error('parse image fail')
        return re_search.group(1)



"""
图床

使用方式:
  1. 指定仓库上传文件
    d = image_depot(DepotType.CatBox) # 获取仓库对象
    if d is None:  # 当前图床已失效
        pass
    url = d.upload(image_content) #  上传二进制内容
    url = d.upload_file(file_path) # 上传文件
    if not url:  # 图片上传失败, 获取失败原因
        print(d.error())
  2. 依次尝试所有仓库, 返回首个上传成功的链接
    url, err = upload(image_content)
    url, err = upload_file(file_path)

配置:
  部分仓库需要进行配置才能使用, 例如配置 sm.ms 图床
    config = DepotConfig()
    config.sm_ms.token = ''
    # 全局配置
    set_global_config(config)
    # 单次上传配置
    d = image_depot(DepotType.CatBox)
    d.set_config(config)

"""
from .depot_type import DepotType
from .image_depot import image_depot, upload, upload_file
from .depot import DepotConfig, set_global_config, get_config_by_dict




from typing import Optional
import requests
from image_depot import DepotType
from .base import Depot


class PomfSe(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.PomfSe

    def _upload_file(self, upload_url: str, file_name: str, content):
        files = {
            'files[]': (file_name, content),
        }
        try:
            response = requests.post(upload_url, files=files)
        except Exception as e:
            # 因为这里会访问多个上传地址, 这次失败了继续下一个
            # 所以要把异常自己消化掉
            return self._set_error(e)

        if response.status_code != 200:
            return self._set_error(f'upload fail. code: {response.status_code}. content: {response.text}')
        data = response.json()
        url = None
        rep_files = data.get('files', [])
        if len(rep_files) == 1:
            url = rep_files[0].get('url')
        if not data.get('success') != 200 or not url:
            return self._set_error(response.text)
        return url

    def _upload(self, content) -> Optional[str]:
        conf = self._config.pomf_se
        if len(conf.upload_url_list) <= 0:
            return self._set_error('No service is available')
        file_name = self._random_file_name(content)
        if not file_name:
            return None
        for upload_url in conf.upload_url_list[:]:
            url = self._upload_file(upload_url, file_name, content)
            if url:
                return url
            else:  # 上传失败, 从列表中去掉, 下次上传可以跳过
                conf.upload_url_list.remove(upload_url)
        return None
